<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Perl Wiki Tree</title>
    <meta http-equiv = 'Content-Type'	content = 'text/html; charset=utf-8' />
	<meta name = 'language'				content = 'en-au'>
	<meta property = 'og:url'			content = 'http://!domain_name!/misc/!page_name!'>
	<meta property = 'og:type'			content = 'website'>
	<meta property = 'og:title	'		content = 'CPAN::MetaCurator'>
	<meta property = 'og:description'	content = 'CPAN::MetaCurator'>
	<meta property = 'og:image'			content = '/assets/images/local/rons.logo.png'>
	<link rel = 'stylesheet'			type = 'text/css' href = '/assets/css/local/default.css'>
	<link rel = 'stylesheet'			type = 'text/css' href = '/assets/css/datatables.min.css'>
	<link rel = 'stylesheet'			type = 'text/css' href = '/assets/css/dist/themes/default/style.min.css'>
<!--
	<link rel = 'stylesheet'			type = 'text/css' href = '/assets/css/hover/hover.1.css'>
-->
	<script src = '/assets/js/jquery-3.7.1.min.js'></script>
	<script src = '/assets/js/jquery-ui-1.14.1.min.js'></script>
	<script src = '/assets/js/jstree.min.js'></script>
  </head>
  <body>
	<a id = 'top'></a>
<div>
	<span><img src = '!logo_path!' width = '120' height = '120' alt = 'Logo for page: index'></span>
	<span class = 'page_title'>A JSTree version of my Perl.Wiki</span>
</div>
<br>
<p>
	!module! V !version!. !time!
</p>
<p>
	<a href = 'http://savage.net.au/'>My home page, for downloading Perl.Wiki.html</a>
</p>
<p>
	Notes:<br />
	<span class = 'local_toc_text'>&#8853;</span> Nodes just under the root ('MetaCurator') are called Topics. Paragraphs is the TiddlyWiki equivalent.<br />
	<span class = 'local_toc_text'>&#8853;</span> Tokens like [[XS]] indicate a TiddlyWiki paragraph name, not a Perl module or topic.<br />
	<span class = 'local_toc_text'>&#8853;</span> Almost all leaf node are links. Use a middle-button click to open the link in another tab.<br />
	<span class = 'local_toc_text'>&#8853;</span> The code does not yet cleanly handle special cases such as the FAQ, pre-formatted paras.<br />
	<span class = 'local_toc_text'>&#8853;</span> Also, cases where the para is just a 'See also' are not yet clean.<br /><br />
	<span class = 'local_toc_text'>&#8853;</span> My other module, CPAN::MetaPackages, builds & ships an SQLite db containing 02packages.details.txt.<br />
	<span class = 'local_toc_text'>&#8853;</span> Most recent version of 02packages.details.txt used: !packages_id!.<br />
	<span class = 'local_toc_text'>&#8853;</span> Acronyms: !acronym!. Known modules: !packages!. Leaves: !leaf!. Topics: !topic!. Unknown modules: !unknown!
</p>
<hr>
