<div id = 'footer'></div>
<script>
	fetch('/misc/footer.html')
	.then(response => response.text())
	.then(data =>
	{
		document.getElementById('footer').innerHTML = data;
	});
</script>
</body>
</html>
